import { Path } from '../path';
import { ReadState, WriteState } from '../state';
import { FeatureMetadata, NoFeature, PartialToken, Token, TokenExtension, _metadata } from '../token';
import { ErrorMessage, ErrorsState } from './error-message';
export declare type ValidationStatusValues = 'pending' | 'validating' | 'invalid' | 'valid';
declare const _validationStatusToken: unique symbol;
export interface ValidationStatus {
    readonly [_validationStatusToken]: Token<ReadState<ValidationStatusValues> & WriteState<ValidationStatusValues>>;
    [_metadata]: FeatureMetadata<'validationStatus', ValidationStatusValues, NoFeature, {
        [P in PropertyKey]: ValidationStatusValues;
    }>;
}
export interface Validated extends Path, ErrorMessage, ValidationStatus {
    readonly [_validationStatusToken]: Token<ReadState<ValidationStatusValues> & WriteState<ValidationStatusValues>>;
    [_metadata]: FeatureMetadata<'validation', Validated, Path & ErrorMessage & ValidationStatus, {
        [P in PropertyKey]: Validated;
    }>;
}
export declare type StateValidator<TState> = (value: TState | undefined) => Promise<ErrorsState>;
export interface ValidationOptions<TState> {
    validator: StateValidator<TState>;
    debounceMs?: number;
}
export declare function addValidation<TState>(options: ValidationOptions<TState>): TokenExtension<ReadState<TState>, Path & ErrorMessage & Validated>;
export declare function getTokenValidationStatus(token: PartialToken<ValidationStatus>): ValidationStatusValues;
export declare function useTokenValidationStatus(token: PartialToken<ValidationStatus>): ValidationStatusValues;
export declare function useTokenValidationStatus(token: PartialToken<ValidationStatus> | undefined): ValidationStatusValues | undefined;
export {};
