import { Dispatch } from 'react';
import { FeatureMetadata, NoFeature, PartialToken, Token, TokenExtension, _metadata } from '../token';
import { NonFunctionProperties } from '../utils/types';
declare const _getValue: unique symbol;
declare const _subscribe: unique symbol;
declare const _setValue: unique symbol;
export interface State<TReadState, TWriteState = TReadState> extends ReadState<TReadState>, WriteState<TWriteState> {
    [_metadata]: FeatureMetadata<'state', State<TReadState, TWriteState>, ReadState<TReadState> & WriteState<TWriteState>, {
        [P in keyof NonFunctionProperties<NonNullable<TReadState>> & keyof NonNullable<TWriteState>]-?: State<NonNullable<TReadState>[P] | (TReadState extends null | undefined ? undefined : never), NonNullable<TWriteState>[P]>;
    }>;
}
export interface ReadState<TState> {
    [_getValue](): TState;
    [_subscribe](listener: TokenValueListener<TState>): Disposer;
    [_metadata]: FeatureMetadata<'readState', ReadState<TState>, NoFeature, {
        [P in keyof NonFunctionProperties<NonNullable<TState>>]-?: ReadState<NonNullable<TState>[P] | (TState extends null | undefined ? undefined : never)>;
    }>;
}
export interface WriteState<TState> {
    [_setValue](newValue: TState): void;
    [_metadata]: FeatureMetadata<'writeState', WriteState<TState>, NoFeature, {
        [P in keyof NonFunctionProperties<NonNullable<TState>>]-?: WriteState<NonNullable<TState>[P]>;
    }>;
}
export declare type TokenValueListener<TState> = (newValue: TState) => void;
export declare type Disposer = () => void;
export declare function addState<TState>(initialValue: TState): TokenExtension<NoFeature, State<TState>>;
export declare function getTokenValue<TState>(token: PartialToken<ReadState<TState>>): TState;
export declare function setTokenValue<TState>(token: PartialToken<WriteState<TState>>, newValue: TState): void;
export declare function subscribeToTokenValue<TState>(token: PartialToken<ReadState<TState>>, listener: TokenValueListener<TState>): Disposer;
export declare function useStateToken<T>(initializer: T | (() => T)): Token<State<T>>;
export declare function useTokenValue<TState>(token: PartialToken<ReadState<TState>>): TState;
export declare function useTokenValue<TState>(token: PartialToken<ReadState<TState>> | undefined): TState | undefined;
export declare function useTokenSetter<TState>(token: PartialToken<WriteState<TState>>): Dispatch<TState>;
export declare function useTokenSetter<TState>(token: PartialToken<WriteState<TState>> | undefined): Dispatch<TState> | undefined;
export declare function useTokenState<TReadState, TWriteState = TReadState>(token: PartialToken<ReadState<TReadState> & WriteState<TWriteState>>): [TReadState, Dispatch<TWriteState>];
export declare function useTokenState<TReadState, TWriteState = TReadState>(token: PartialToken<ReadState<TReadState> & WriteState<TWriteState>> | undefined): [TReadState | undefined, Dispatch<TWriteState> | undefined];
export {};
