declare const _children: unique symbol;
declare const _childrenInterceptor: unique symbol;
export declare const _metadata: unique symbol;
interface TokenBase {
    [_children]?: Record<PropertyKey, any>;
    [_childrenInterceptor]: ChildTokenInterceptor;
}
export interface NoFeature {
}
export declare type PartialToken<Features> = TokenBase & Omit<Features, typeof _metadata>;
export declare type Token<Features> = PartialToken<Features> & ChildTokensProperties<Features>;
declare type ChildTokensProperties<Features> = {
    readonly [k in keyof ChildFeaturesProperties<Features>]: Token<CleanupFeatures<ChildFeaturesProperties<Features>[k]>>;
};
export declare function createToken(): Token<NoFeature>;
export declare function createToken<Feature>(extension?: TokenExtension<NoFeature, Feature>): Token<CleanupFeatures<NoFeature & Feature>>;
export declare function extendToken<CurrentFeatures extends BaseFeature, BaseFeature, NewFature>(token: Token<CurrentFeatures>, extension: TokenExtension<BaseFeature, NewFature>): Token<CleanupFeatures<CurrentFeatures & NewFature>>;
export declare function restoreToken<Features>(partialToken: PartialToken<Features>): Token<Features>;
export declare type TokenExtension<BaseFeatures, NewFeatures> = {
    extend: Omit<PartialToken<NewFeatures>, keyof PartialToken<BaseFeatures>>;
    extendChildren?: boolean | ((token: Token<BaseFeatures>, parentToken: Token<BaseFeatures & NewFeatures>, property: PropertyKey) => Omit<PartialToken<NewFeatures>, keyof PartialToken<BaseFeatures>>);
} | ((token: Token<BaseFeatures>) => Token<CleanupFeatures<BaseFeatures & NewFeatures>>);
export declare type FeatureMetadata<UniqueIdentifier extends string, Self, BaseFeatures, ChildFeatures> = {
    self: BaseFeatures extends {
        [K in typeof _metadata]?: {
            self: infer S;
        };
    } ? S & {
        [U in UniqueIdentifier]?: Self;
    } : {
        [U in UniqueIdentifier]?: Self;
    };
    baseFeatures: BaseFeatures extends {
        [K in typeof _metadata]?: {
            baseFeatures: infer B;
        };
    } ? B & {
        [U in UniqueIdentifier]?: BaseFeatures;
    } : {
        [U in UniqueIdentifier]?: BaseFeatures;
    };
    childFeatures: BaseFeatures extends {
        [K in typeof _metadata]?: {
            childFeatures: infer C;
        };
    } ? C & {
        [U in UniqueIdentifier]?: ChildFeatures;
    } : {
        [U in UniqueIdentifier]?: ChildFeatures;
    };
};
declare type GetIdentifiers<Features> = Features extends {
    [K in typeof _metadata]?: {
        self: infer T;
    };
} ? keyof T : never;
declare type GetNonBaseIdentifiers<Features> = Exclude<GetIdentifiers<Features>, GetIdentifiers<GetBaseFeatures<Features>>>;
declare type GetBaseFeatures<Features> = UnionToIntersection<Features extends {
    [C in typeof _metadata]?: {
        baseFeatures: {
            [U in string]?: infer Base;
        };
    };
} ? Base : never>;
declare type CleanupFeatures<Features> = UnionToIntersection<Features extends {
    [C in typeof _metadata]?: {
        self: {
            [U in GetNonBaseIdentifiers<Features>]?: infer Self;
        };
    };
} ? Self : never>;
declare type ChildFeaturesProperties<Features> = UnionToIntersection<Features extends {
    [K in typeof _metadata]?: {
        childFeatures: {
            [U in string]?: infer T;
        };
    };
} ? T : never>;
declare type ChildTokenInterceptor = (token: any, parentToken: any, property: PropertyKey) => any;
declare type UnionToIntersection<T> = (T extends any ? (x: T) => any : never) extends (x: infer R) => any ? R : never;
export declare function useToken(): Token<NoFeature>;
export declare function useToken<Feature>(extensionFactory?: () => TokenExtension<NoFeature, Feature>): Token<Feature>;
export declare function useTokenExtension<CurrentFeatures extends BaseFeatures, NewFeatures, BaseFeatures = CurrentFeatures>(token: Token<CurrentFeatures>, extensionFactory: () => TokenExtension<BaseFeatures, NewFeatures>): Token<NewFeatures extends CurrentFeatures ? NewFeatures : CurrentFeatures & NewFeatures>;
export {};
